"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectImportNames = exports.collectImportInfo = void 0;
const tslib_1 = require("tslib");
const ts = tslib_1.__importStar(require("typescript"));
const util_cjs_1 = require("./util.cjs");
function listNamesOfImportLine(node) {
    const ret = [];
    const { importClause } = node;
    if (!importClause) {
        return ret;
    }
    if (importClause.name) {
        // import def from
        ret.push({ identifier: importClause.name, name: 'default' });
    }
    if (importClause.namedBindings) {
        const { namedBindings } = importClause;
        if (ts.isNamedImports(namedBindings)) {
            // import {a as b, c} from
            for (const item of namedBindings.elements) {
                const id = item.propertyName || item.name;
                ret.push({ identifier: id, name: util_cjs_1.nameToString(id) });
            }
        }
        else if (ts.isNamespaceImport(namedBindings)) {
            // import * as all from
            ret.push({ identifier: namedBindings.name, name: '*' });
        }
    }
    return ret;
}
function collectImportInfo(sourceFile, nodes, typeChecker) {
    const ret = {
        types: [],
        values: [],
    };
    // console.log('='.repeat(process.stderr.columns) + '\n', sourceFile.fileName);
    const symbolsNeedCheck = new Map();
    for (const node of nodes) {
        for (const value of listNamesOfImportLine(node)) {
            const type = typeChecker.getTypeAtLocation(value.identifier);
            if (type.getSymbol()) {
                symbolsNeedCheck.set(type, value);
            }
            else {
                ret.types.push(value.name);
            }
        }
    }
    // for (const [_, symbolIdentifier] of symbolsNeedCheck.entries()) {
    // 	dumpNode(symbolIdentifier);
    // }
    ts.forEachChild(sourceFile, function tokenWalk(node) {
        if (ts.isImportDeclaration(node)) {
            return;
        }
        if (ts.isTypeQueryNode(node)) {
            // console.error('[SKIP]', ts.SyntaxKind[node.kind], node.getText());
            return;
        }
        if (ts.isVariableDeclaration(node)) {
            if (node.initializer)
                tokenWalk(node.initializer);
            return;
        }
        if (ts.isIdentifier(node)) {
            // console.error('[%s]', ts.SyntaxKind[node.kind], node.getText());
            const symbol = typeChecker.getSymbolAtLocation(node);
            if (symbol) {
                const type = typeChecker.getTypeAtLocation(node);
                const importedSymbol = symbolsNeedCheck.get(type);
                if (importedSymbol) {
                    symbolsNeedCheck.delete(type);
                    // console.log('!!!!', nameToString(importedSymbol), typeChecker.getFullyQualifiedName(symbol));
                    ret.values.push(importedSymbol.name);
                }
            }
            return;
        }
        ts.forEachChild(node, tokenWalk);
    });
    for (const { name } of symbolsNeedCheck.values()) {
        ret.types.push(name);
    }
    return ret;
}
exports.collectImportInfo = collectImportInfo;
/** @deprecated */
function collectImportNames(node) {
    const ret = [];
    if (ts.isImportDeclaration(node)) {
        const { importClause } = node;
        if (!importClause) {
            return ret;
        }
        if (importClause.name) {
            // import def from
            ret.push('default');
        }
        if (importClause.namedBindings) {
            const { namedBindings } = importClause;
            if (ts.isNamedImports(namedBindings)) {
                // import {a as b, c} from
                ret.push(...getAllNames(namedBindings.elements));
            }
            else if (ts.isNamespaceImport(namedBindings)) {
                // import * as all from
                ret.unshift('*');
            }
        }
    }
    else if (ts.isExportDeclaration(node)) {
        const { exportClause } = node;
        if (!exportClause) {
            // export * from
            return ['*'];
        }
        if (ts.isNamedExports(exportClause)) {
            // export {a as b, c} from
            ret.push(...getAllNames(exportClause.elements));
        }
    }
    return ret;
}
exports.collectImportNames = collectImportNames;
function getAllNames(list) {
    return list.map((v) => util_cjs_1.nameToString(v.propertyName || v.name));
}
//# sourceMappingURL=importIds.js.map